if [namespace exists Cmd::ListEdit] return

namespace eval Cmd {
	namespace eval ListEdit {}
}
source renametag_dialog.tcl

proc Cmd::ListEdit::buildMenu {parent} {
	set m $parent.listedit 
	if {[winfo exists $m]} { return $m }
	menu $m -tearoff 0

	menu $m.check    -tearoff 0
	menu $m.uncheck  -tearoff 0
	menu $m.invcheck -tearoff 0
	foreach {status name} {
		all S normal Normal edit Edit new New update Update 
		delete Delete notfound NotFound error Error} {
		set icon [::Tree::getStatusImage $status]
		$m.check add command -label $name -compound left \
		-command "Cmd::ListEdit::checkStatus $status on" -image $icon
		$m.uncheck add command -label $name -compound left \
		-command "Cmd::ListEdit::checkStatus $status off" -image $icon
		$m.invcheck add command -label $name -compound left \
		-command "Cmd::ListEdit::checkStatus $status inv" -image $icon
	}
	$m add cascade -label `FbN   -menu $m.check
	$m add cascade -label `FbNȂ -menu $m.uncheck
	$m add cascade -label `FbN𔽓] -menu $m.invcheck

#    $m add separator
#    $m add command -label ^OŃ`FbN...
	$m add separator
	$m add command -label 폜 \
		-command "Cmd::ListEdit::remove"
	$m add separator
	$m add command -label ^O... \
		-command "Cmd::ListEdit::addTag ::doc"
	$m add command -label ^O폜... \
		-command "Cmd::ListEdit::removeTag ::doc"
	$m add command -label ^Õl[... \
		-command "Cmd::ListEdit::renameTag ::doc"
#    $m add command -label d^O̍폜
#    $m add command -label Rg̍폜
	return $m
}

#=============================================================
# ^OARgҏW֘A
#=============================================================
# V^O
proc Cmd::ListEdit::addTag {docName} {
	upvar $docName doc
	if {$doc eq ""} return

	Prompt::show "^O͂Ă" "V^O͂Ă" ""
	if {$Prompt::result ne "ok"} return
	if {$Prompt::str eq ""} return

	set ntag $Prompt::str

	# table list
	set donothinglist [list]
	foreach item [Tree::getAll] {
		foreach {check status title url summary date} [Tree::getItemData $item] break

		if {$check == 0} {
			lappend donothinglist $url
			continue
		}
		foreach {taglist comment} \
		[Hatena::Bookmark::getTagComment $summary] break
		
		lappend taglist $ntag
		set summary [Hatena::Bookmark::buildSummary $taglist $comment]
		Tree::updateItem $item $check edit $title $url $summary $date
	}
	# dom tree
	set feed [$doc firstChild]
	set xmlns [$feed @xmlns:dc]
	foreach n [$feed getElementsByTagName entry] {
		set url ""
		foreach ln [$n getElementsByTagName "link"] {
			if {[$ln @rel] eq "related"} { set url [$ln @href]; break }
		}
		if {[lsearch $donothinglist $url] >= 0} continue
		
		set new [$doc createElementNS $xmlns "dc:subject"]
		$new appendChild [$doc createTextNode $ntag]

		set lastcn [lindex [$n getElementsByTagName "dc:subject"] end]
		if {$lastcn eq ""} {
			$n appendChild $new
			continue
		}

		set nextcn [$lastcn nextSibling]
		if {$nextcn eq ""} {
			$n appendChild $new
		} else {
			$n insertBefore $new $next
		}
	}
	return
}
# ^O폜
proc Cmd::ListEdit::removeTag {docName} {
	upvar $docName doc
	if {$doc eq ""} return

	Prompt::show "^O͂Ă" "폜^O͂Ă" ""
	if {$Prompt::result ne "ok"} return

	set dtag $Prompt::str

	# table list
	set donothinglist [list]
	foreach item [Tree::getAll] {
		foreach {check status title url summary date} [Tree::getItemData $item] break
		
		if {$check == 0} {
			lappend donothinglist $url
			continue
		}
		foreach {taglist comment} \
		[Hatena::Bookmark::getTagComment $summary] break
		
		set found 0
		set ntaglist [list]
		foreach tag $taglist {
			if {$tag eq $dtag} {
				set found 1
				continue
			}
			lappend ntaglist $tag
		}
		if {$found == 0} {
			lappend donothinglist $url
			continue
		}
		set taglist $ntaglist
		
		set summary [Hatena::Bookmark::buildSummary $taglist $comment]
		Tree::updateItem $item $check edit $title $url $summary $date
	}
	# dom tree
	set feed [$doc firstChild]
	set xmlns [$feed @xmlns:dc]
	foreach n [$feed getElementsByTagName entry] {
		set url ""
		foreach ln [$n getElementsByTagName "link"] {
			if {[$ln @rel] eq "related"} { set url [$ln @href]; break }
		}
		if {[lsearch $donothinglist $url] >= 0} continue

		foreach tn [$feed getElementsByTagName dc:subject] {
			if {[$tn text] eq $dtag} {
				$tn delete
			}
		}
	}
	return
}
# ^Õl[
proc Cmd::ListEdit::renameTag {docName} {
	upvar $docName doc
	if {$doc eq ""} return
	
	Renametag::show
	if {$Renametag::result ne "ok"} return
	
	set srct $Renametag::from
	set dstt $Renametag::to

	# table list
	set donothinglist [list]
	foreach item [Tree::getAll] {
		foreach {check status title url summary date} [Tree::getItemData $item] break
		if {$check == 0} {
			lappend donothinglist $url
			continue
		}
		foreach {taglist comment} \
		[Hatena::Bookmark::getTagComment $summary] break
		
		set ntaglist [list]
		foreach tag $taglist {
			lappend ntaglist [expr {$tag eq $srct ? $dstt : $tag }]
		}
		set taglist $ntaglist
		
		set summary [Hatena::Bookmark::buildSummary $taglist $comment]
		Tree::updateItem $item $check edit $title $url $summary $date
	}
	
	# dom tree
	set feed [$doc firstChild]
	set xmlns [$feed @xmlns:dc]
	foreach n [$feed getElementsByTagName entry] {
		set url ""
		foreach ln [$n getElementsByTagName "link"] {
			if {[$ln @rel] eq "related"} { set url [$ln @href]; break }
		}
		if {[lsearch $donothinglist $url] >= 0} continue

		foreach tn [$feed getElementsByTagName dc:subject] {
			if {[$tn text] eq $srct} {
				[$tn firstChild] delete
				$tn appendChild [$doc createTextNode $dstt]
			}
		}
	}
	return
}

#=============================================================
# e[uXg
#=============================================================
proc Cmd::ListEdit::remove {} {
	
	set itemlist [list]
	foreach item [::Tree::getAll] {
		if {[::Tree::isChecked $item] == 0} continue
		lappend itemlist $item
	}
	_removeEntry $itemlist
}
#=============================================================
# `FbNO
#=============================================================
proc Cmd::ListEdit::checkStatus {status check} {
	if {$status eq "all"} {
		foreach item [::Tree::getAll] {
			::Tree::checkItem $item $check
		}
		return
	}
	
	foreach item [::Tree::getAll] {
		if {[::Tree::getStatus $item] eq $status} {
			::Tree::checkItem $item $check
		}
	}
}
