if [namespace exists Hatena::Bookmark] return
source hatena.tcl

namespace eval Hatena {
	namespace eval Bookmark {
		array set uri {
			post ""
			feed ""
		}
		variable trycount 5
	}
}

#=============================================================
# c[
#=============================================================
# ^OXgNG𐶐
proc Hatena::Bookmark::tagquery {taglist} {
	set qlist [list]
	foreach tag $taglist {
		lappend qlist tag $tag
	}
	return [eval http::formatQuery $qlist]
}

# Gg̃m[h"[tag][tag]summary"`̕ԂB
proc Hatena::Bookmark::getCommentFromEntry {en} {
	set str ""
	foreach n [$en getElementsByTagName "dc:subject"] {
		append str \[[$n text]\]
	}
	append str [[$en getElementsByTagName "summary"] text]
	return $str
}
# ^OXgƃRgT}[𐶐
proc Hatena::Bookmark::buildSummary {taglist comment} {
	set summary ""
	foreach tag $taglist {
		append summary \[$tag\]
	}
	append summary $comment
	return $summary
}
# T}[^OXgƃRg𒊏o
proc Hatena::Bookmark::getTagComment {summary} {
	if {[string index $summary 0] ne "\["} {
		return [list {} $summary]
	}
	set st 0
	set len [string length $summary]
	set taglist [list]
	while {$st < $len} {
		set ed [string first "\]" $summary $st]
		if {$ed < 0} {
			set comment [string range $summary $st end]
			break
		}
		incr st
		incr ed -1
		lappend taglist [string range $summary $st $ed]
		incr ed +2
		if {[string index $summary $ed] ne "\["} {
			set comment [string range $summary $ed end]
			break
		}
		set st $ed
	}
	return [list $taglist $comment]
}
#=============================================================
# ڑ
# Ԓl̂DOMc[IuWFNgԂBgI
# deleteŃ邱ƁB
#=============================================================
proc Hatena::Bookmark::connect {} {
	variable uri
	set data ""
	set code [Hatena::atom "http://b.hatena.ne.jp/atom" GET "" data]
	if {$code != 200} { error "[info level [info level]]:$code" }
	set doc [dom parse $data]
	set root [$doc firstChild]
	foreach child [$root childNodes] {
		switch -- [$child @rel] {
		"service.post" {set uri(post) [Util::rootURI [$child @href]]}
		"service.feed" {set uri(feed) [Util::rootURI [$child @href]]}
		}
	}
	$doc delete
	return
}


#VKeBe邱ƂłȂB
proc Hatena::Bookmark::postEntry {url summary} {
	variable uri
	variable trycount
	set data ""

	set entry    "<?xml version=\"1.0\" encoding=\"utf-8\"?>"
	append entry "<entry xmlns=\"http://purl.org/atom/ns#\">"
	append entry "<link rel=\"related\" type=\"text/html\" href=\"$url\" />"
	append entry "<summary type=\"text/plain\">$summary</summary>"
	append entry "</entry>\n"
	for {set i 0} {$i < $trycount} {incr i} {
		set code [Hatena::atom "$uri(post)/atom/post" POST $entry data]
		if {$code == 201} {
			return [dom parse $data]
		}
	}
	error "[info level [info level]]:$code"
}


# Gg[̎擾
proc Hatena::Bookmark::getEntryFromEid {eid} {
	variable uri
	variable trycount
	set data ""
	for {set i 0} {$i < $trycount} {incr i} {
		set code [Hatena::atom "$uri(post)/atom/edit/$eid" GET "" data]
		if {$code == 200} {
			return [dom parse $data]
		}
	}
	error "[info level [info level]]:$code"
}

# url̃Gg[擾Blo^Ă΁Ao^ĂȂ̂ł
#eidȂǂ擾邱Ƃł悤Bo^ĂȂꍇ<issued>̂悤B
proc Hatena::Bookmark::getEntryFromUrl {url} {
	variable uri
	variable trycount
	set data ""
	set query [http::formatQuery url $url]
	for {set i 0} {$i < $trycount} {incr i} {
		set code [Hatena::atom "$uri(post)/atom/edit?$query" GET "" data]
		if {$code == 200} {
			dputs $data
			return [dom parse $data]
		}
	}
	error "[info level [info level]]:$code"
}

# Gg[̏B^Cg̕ύX͎dl\ǁAȂjŁB
proc Hatena::Bookmark::putEntry {eid summary} {
	variable uri
	variable trycount
	set data ""

	set entry ""
	append entry "<entry xmlns=\"http://purl.org/atom/ns#\">"
	append entry "<summary type=\"text/plain\">$summary</summary>"
	append entry "</entry>"
	
	for {set i 0} {$i < $trycount} {incr i} {
		set code [Hatena::atom "$uri(post)/atom/edit/$eid" PUT $entry data]
		if {$code == 200} return
	}
	error "[info level [info level]]:$code"
}


# 폜
proc Hatena::Bookmark::deleteEntry {eid} {
	variable uri
	variable trycount
	set data ""
	for {set i 0} {$i < $trycount} {incr i} {
		set code [Hatena::atom "$uri(post)/atom/edit/$eid" DELETE "" data]
		if {$code == 200} return
	}
	error "[info level [info level]]:$code"
}


# ŋ߂̂擾
proc Hatena::Bookmark::feedRecent {{taglist {}}} {
	variable root
	variable trycount
	set data ""
	set query ""
	if {[llength $taglist] > 0} {
		append query "?" [tagquery $taglist]
	}
	for {set i 0} {$i < $trycount} {incr i} {
		set code [Hatena::atom "$uri(feed)/atom/feed$query" GET "" data]
		if {$code == 200} {
			return [dom parse $data]
		}
	}
	error "[info level [info level]]:$code"
}


# SĎ擾
# callback͈ɓKȃXgnBǂ̂悤Ȉ
# 󂯎鑤ŏ邱ƁBcallback0ȊOԂ璆~B
proc Hatena::Bookmark::feedAll { {taglist ""} {callback ""} } {
	variable uri
	variable trycount
	set data ""
	set query ""
	if {[llength $taglist] > 0} {
		append query "&" [tagquery $taglist]
	}

	set total -1
	set index  0
	set len  0
	set of 0
	set doc ""
	set feed ""
	while {1} {
		for {set i 0} {$i < $trycount} {incr i} {
			set code [Hatena::atom "$uri(feed)/atom/feed?of=$of$query" GET "" data]
			if {$code == 200} break
		}
		if {$code != 200} { error "[info level [info level]]:$code" }

		set tmpdoc [dom parse $data]
		set entrylen 0
		if {$doc eq ""} {
			set doc $tmpdoc
			set feed [$doc firstChild]
			set entrylen [llength [$doc getElementsByTagName entry]]
			set ntotal [$doc getElementsByTagName openSearch:totalResults]
			if {$ntotal eq ""} {
				# y[WꖇɎ܂ĂꍇopenSearch:`炵B
				return $doc
			} else {
				set total [$ntotal text]
				set st    [[$doc getElementsByTagName openSearch:startIndex] text]
				set len   [[$doc getElementsByTagName openSearch:itemsPerPage] text]
			}
		} else {
			set tmpfeed [$tmpdoc firstChild]
			foreach n [$tmpfeed childNodes] {
				if {[$n nodeName] ne "entry"} continue
				$feed appendChild [$tmpfeed removeChild $n]
				incr entrylen
			}
			set total [[$tmpdoc getElementsByTagName openSearch:totalResults] text]
			set st    [[$tmpdoc getElementsByTagName openSearch:startIndex] text]
			set len   [[$tmpdoc getElementsByTagName openSearch:itemsPerPage] text]
			$tmpdoc delete
		}
		if {[info command $callback] ne ""} {
			set cb_idx [expr {$st+$len}]
			if {[$callback 0 $total $cb_idx "$cb_idx / $total"] != 0} {
				# stop
				[$feed getElementsByTagName openSearch:itemsPerPage] delete
				return $doc
			}
		}
		if {$entrylen == 0 || $entrylen < $len} break
		set of [expr {$st + $len -1}]
#        after 2000
		::Util::timer 2000 ::Progress::result
	}
	if {[info command $callback] ne ""} { $callback 0 $total $total "$total / $total" }
	[[$feed getElementsByTagName openSearch:itemsPerPage] firstChild] nodeValue $total
	return $doc
}
