if [namespace exists Util] return


namespace eval Util {;#<<<
	array set lang {
		cut   Jbg
		copy  Rs[
		paste y[Xg
		delete 폜
		select_all SđI
	}
	# Is Number?
	proc isNumber {str} {
		if {$str eq {}} {return 0}
		if {[string is integer $str]} {return 1}
		if {[string is double $str]} {return 1}
		return 0
	}
	
	# lintersect {a b c d e} {c d e f g} returns {c d e}
	proc lintersect {a b} {
		if {[llength $a]==0 || [llength $b]==0} {return [list]}
		if {[llength $a] > [llength $b]} {
			set L $a
			set S $b
		} else {
			set L $b
			set S $a
		}
		set S [lsort -uniq $S]
		set ret [list]
		foreach e $S {
			set n [lsearch -exact $L $e]
			if {$n >= 0} {
				set L [lreplace $L $n $n]
				lappend ret $e
			}
		}
		return $ret
	}
	
	# increment x,y format. incrIndex 0,0 , 1 0
	proc incrIndex {index sep {xi 0} {yi 0}} {
		set l [split $index $sep]
		foreach {x y} $l {
			if {$x eq {}} {set x 0}
			if {$y eq {}} {set y 0}
			incr x $xi
			incr y $yi
			return $x$sep$y
		}
	}
	
	# parse sequens number
	# {1 2 3 4 7 8 9} is converted to {{1 4} {7 3}}.
	proc exactConsecutiveNumber {inlist} {
		if {[llength $inlist] == 0} return
		set nlist [lsort -integer -uniq $inlist]
		set ret [list]
		
		set seqS [lindex $nlist 0]
		set i $seqS
		foreach n [lrange $nlist 1 end] {
			incr i
			if {$n != $i} {
				lappend ret [list $seqS [expr {$i - $seqS}]]
				set seqS $n
				set i    $n
			}
		}
		incr i
		lappend ret [list $seqS [expr {$i - $seqS}]]
		return $ret
	}
	

	# Python's range like
	proc range {from to {step 1}} {
		set res $from
		while {$to>$from} {lappend res [incr from $step]}
		return $res
	}
	
	
	# make pop and return it.
	# $root.pop   PopMenu path (created here)
	# $root.text  Text Widget path
	proc makeEditorPop {root} {
		variable lang
		set m $root.pop
		if ![winfo exists $m] { menu $m }
		$m add command -label $lang(cut) -accelerator {Ctrl+X} \
			-command "tk_textCut $root.text"
		$m add command -label $lang(copy) -accelerator {Ctrl+C} \
			-command "tk_textCopy $root.text"
		$m add command -label $lang(paste) -accelerator {Ctrl+P} \
			-command "tk_textPaste  $root.text"
		$m add command -label $lang(delete) -accelerator {Ctrl+D} \
		-command "event generate $root.text <Control-d>"
		$m add separator
		$m add command -label $lang(select_all) -accelerator {Ctrl+/} \
			-command "event generate $root.text <Control-slash>"
		return $m
	}
	
	# bind to common pop.
	proc bindEditorPopCommon {w} {
		variable lang
		set m ._editorpop
		if ![winfo exists $m] {
			menu $m -tearoff 0
			$m add command -label $lang(cut) -accelerator {Ctrl+X} \
			-command {event generate [focus] <<Cut>>}
			$m add command -label $lang(copy) -accelerator {Ctrl+C} \
			-command {event generate [focus] <<Copy>>}
			$m add command -label $lang(paste) -accelerator {Ctrl+V} \
			-command {event generate [focus] <<Paste>>}
			$m add command -label $lang(delete) -accelerator {Ctrl+D} \
			-command {event generate [focus] <Control-d>}
			$m add separator
			$m add command -label $lang(select_all) -accelerator {Ctrl+/} \
			-command {event generate [focus] <Control-slash>}
		}
		bind $w <3> "if {\[$w cget -state\] ne \"disabled\"} {
					focus $w; tk_popup $m %X %Y}"
	}
	
	proc quoteValue {val nullString} {
		if {$val eq $nullString} {
			return "NULL"
		}
		if {[isNumber $val]} {
			return $val
		}
		return "'$val'"
	}

	proc moveCenter {widget {size {}}} {
		set sw [winfo vrootwidth $widget]
		set sh [winfo vrootheight $widget]
		if {[llength $size] == 0} {
			update idletask
			set w [winfo width $widget]
			set h [winfo height $widget]
		} else {
			set w [lindex $size 0]
			set h [lindex $size 1]
		}
		wm geometry $widget +[expr {($sw-$w)/2}]+[expr {($sh-$h)/2}]
	}
	
	# Nested grab command
	# ngrab set .w   : stack window and grab it.
	# ngrab release  : release a top of stack window.
	variable grabStack [list]
	proc ngrab {command {w {}}} {
		variable grabStack
		switch -exact -- $command {
			set {
				if {$w eq {}} { error "wrong args : ngrab command window" }
				grab set $w
				focus -force $w
				lappend grabStack $w
			}
			release {
				set cur [lindex $grabStack end]
				set grabStack [lrange $grabStack 0 "end-1"]
				if {[llength $grabStack] == 0} {
					grab release $cur
					return
				}
				grab set [lindex $grabStack end]
			}
		}
	}
	
	# bind mouse event to button widgets
	proc bindMouseEvent {w} {
		bind $w <KeyPress-Return> {
			event generate %W <ButtonPress-1>
		}
		bind $w <KeyRelease-Return> {
			event generate %W <ButtonRelease-1>
		}
	}
	
	# get quoted string. this proc is used for parsing SQLite SQL statement.
	proc _sqlGetQuotedIndices {strName pos} {
		upvar $strName str
		set pos [lindex [regexp -start $pos -indices -inline -- {[^\s]} $str] 0 0]
		switch -exact -- [string index $str $pos] {
			\[ { set q {\[[^\]]*\]} }
			'  { set q {'[^']*'} }
			\" { set q {\"[^\"]*?\"} }
			default {
				set q {[^(\s|\.)]+}
				return [lindex [regexp -start $pos -indices -inline -- $q $str] 0]
			}
		}
		set range [lindex [regexp -start $pos -indices -inline -- $q $str] 0]
		set st [lindex $range 0]
		set ed [lindex $range 1]
		return [list $st $ed]
	}
	
	# return {db name nextpos} list. pos must be top of a word.
	proc _sqlGetSchemaName {strName pos} {
		upvar $strName str
		foreach nn {1 2} {
			set range [_sqlGetQuotedIndices str $pos]
			foreach {st ed} $range {}
			set stchar [string index $str $st]
			if {$stchar eq "\[" || $stchar eq "'" || $stchar eq "\""} {
				incr st; incr ed -1
			}
			set pos [expr {[lindex $range 1]+1}]
			if {$nn == 1} {
				set name1 [string range $str $st $ed]
				if {[string index $str $pos] ne "."} { return [list {} $name1 $pos] }
			} else {
				set name2 [string range $str $st $ed]
				return [list $name1 $name2 $pos]
			}
		}
	}
	
	# remove comment from sql.
	proc _sqlRemoveComment {strName} {
		upvar $strName str
		set str [regsub -all -- {/\*.*?\*/} $str { }]
		set str [regsub -all -line -- {--.*$} $str { }]
	}

	# parser of "CREATE INDEX" statement
	# sql must be correct statement.
	# return {indexName isUniq {colName1 colName2 ...}}
	proc parseCreateIndexStatement {sql} {
		_sqlRemoveComment sql
		set uniq 0
		set uniq [regexp -nocase -- {^\s*create\s+unique\s+index} $sql]

		set pos [lindex [regexp -nocase -inline -indices -- "index" $sql] 0 1]
		incr pos
		set pos [lindex [regexp -start $pos -inline -indices -- {[^\s]} $sql] 0 0]
		foreach {db name pos} [_sqlGetSchemaName sql $pos] {}
		set pos [lindex [regexp -nocase -inline -indices -- "on" $sql] 0 1]
		incr pos
#        foreach {db table pos} [_sqlGetSchemaName sql $pos] {}
		set sql [string range $sql $pos end]
		
		set columns [string range $sql \
			[expr {[string first ( $sql]+1}] \
			[expr {[string last  ) $sql]-1}] ]
		set clist [list]
		foreach col [split $columns ,] {
			set colname [lindex $col 0]
			foreach {dummy col pos} [_sqlGetSchemaName colname 0] {}
			lappend clist $col
		}
		return [list $name $uniq $clist]
	}
	
	# read csv and return tcl list
	proc readCSVLine {fp sep quote {lenmax 4000}} {
		if {[eof $fp]} return
		set str [string trimleft [gets $fp]]
		set len [string length $str]
		set map [list $quote$quote $quote]
		if {$len == 0} return
		set data [list]
		set st 0; set ed 0;
		while 1 {
			set quoted 0
			set stchar [string index $str $st]
			if {$stchar eq $quote} {incr st; set quoted 1}
			if {$stchar eq $sep}   {lappend data {}; incr st; continue}

			if {$quoted} {
				# Quoted
				set ed -1
				set haveQuote 0
				set sst $st
				while {$ed == -1} {
					set ed [string first $quote $str $sst]
					if {$ed == -1} {
						if {[eof $fp]} break
						append str \n[gets $fp]
						set len [string length $str]
					} else {
						if {[string index $str [expr {$ed+1}]] eq $quote} {
							set haveQuote 1
							set sst [expr {$ed+2}]
							set ed -1
						}
					}
				}
				if {$ed == -1} {
					lappend data [string range $str $st end]
					break
				} else {
					incr ed -1
					if {$haveQuote} {
						lappend data [string map $map [string range $str $st $ed]]
					} else {
						lappend data [string range $str $st $ed]
					}
					set st [expr {[string first $sep $str [incr ed 1]] +1}]
					if {$st == 0} break
				}
			} else {
				# Doesn't Quoted
				set ed [string first $sep $str $st]
				if {$ed == -1} {
					lappend data [string trim [string range $str $st end]]
					break
				}
				lappend data [string trim [string range $str $st [expr {$ed-1}]]]
				set st [incr ed]
			}
		}
		return $data
	}
	
	# return root
	proc rootURI {uri} {
		return [join [lrange [split $uri /] 0 2] /]
	}
	
	# output tdom document tree as xml
	proc outputXML {doc file} {
#        $doc omit-xml-declaration no ;# this option is broken?
		set fp [open $file w]
		fconfigure $fp -encoding utf-8
		puts $fp "<?xml version=\"1.0\" encoding=\"utf-8\"?>"
		if {[info command $doc] ne ""} {
		puts $fp [$doc asXML]
		}
		close $fp
	}
	# parse xml and return tdom tree
	proc inputXML {file} {
		set fp [open $file r]
		fconfigure $fp -encoding utf-8
		set doc [dom parse [read -nonewline $fp]]
		close $fp
		return $doc
	}

	# timer. wait msec or change var.
	proc timer {msec var} {
		set nvar $var
		set ac [after $msec set $var $nvar]
		vwait $var
		after cancel $ac
	}
}
;#>>>
